package hu.bme.mit.graph.editor;

import java.io.IOException;
import java.util.HashMap;

import hu.bme.mit.graph.editor.factories.ArcFactory;
import hu.bme.mit.graph.editor.factories.RectangleNodeFactory;
import hu.bme.mit.graph.editor.parts.GraphEditPartFactory;
import hu.bme.mit.graph.model.graph.Graph;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;

/**
 * 
 */

/**
 * @author meres
 * 
 */
public class GraphModelEditor extends GraphicalEditorWithFlyoutPalette {

	Resource resource;
	Graph graph;

	/**
	 * 
	 */
	public GraphModelEditor() {
		setEditDomain(new DefaultEditDomain(this));
	}

	@Override
	protected void initializeGraphicalViewer() {
		super.initializeGraphicalViewer();
		GraphicalViewer viewer = getGraphicalViewer();
		viewer.setEditPartFactory(new GraphEditPartFactory());
		viewer.setContents(graph);
		getEditDomain().addViewer(viewer);

	}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		FileEditorInput fileInput = (FileEditorInput) input;

		ResourceSet set = new ResourceSetImpl();
		resource = set.getResource(URI.createFileURI(fileInput.getPath()
				.toOSString()), true);
		try {
			graph = (Graph) resource.getContents().get(0);
			super.init(site, input);
			ActionRegistry actions = getActionRegistry();
			actions.registerAction(new UndoAction(this));
			actions.registerAction(new RedoAction(this));
		} catch (Exception e) {
			throw new PartInitException("Cannot load model", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette#getPaletteRoot
	 * ()
	 */
	@Override
	protected PaletteRoot getPaletteRoot() {
		PaletteRoot root = new PaletteRoot();
		PaletteGroup selectionToolGroup = new PaletteGroup("Selection");
		ToolEntry tool = new SelectionToolEntry();
		selectionToolGroup.add(tool);
		root.setDefaultEntry(tool);
		tool = new MarqueeToolEntry();
		selectionToolGroup.add(tool);
		root.add(selectionToolGroup);
		root.add(new PaletteSeparator());
		PaletteGroup creationGroup = new PaletteGroup("Creation");
		creationGroup.add(new CreationToolEntry("Node", "Creates a node",
				new RectangleNodeFactory(), null, null));
		creationGroup.add(new ConnectionCreationToolEntry("Arc",
				"Creates an arc", new ArcFactory(), null, null));
		root.add(creationGroup);
		return root;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	@Override
	public void doSave(IProgressMonitor monitor) {
		try {
			resource.save(new HashMap<String, Boolean>());
			getCommandStack().markSaveLocation();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public ActionRegistry getActionRegistry() {
		return super.getActionRegistry();
	}
}
