/**
 * 
 */
package hu.bme.mit.graph.editor.parts;

import java.util.List;

import hu.bme.mit.graph.editor.policies.NodeGraphicalNodeEditPolicy;
import hu.bme.mit.graph.editor.view.AbstractNodeFigure;
import hu.bme.mit.graph.editor.view.CircleNodeFigure;
import hu.bme.mit.graph.editor.view.RectangleNodeFigure;
import hu.bme.mit.graph.model.graph.Arc;
import hu.bme.mit.graph.model.graph.GraphPackage;
import hu.bme.mit.graph.model.graph.Node;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.w3c.dom.traversal.NodeFilter;

/**
 * @author meres
 * 
 */
public class NodeEditPart extends AbstractGraphicalEditPart implements Adapter,
		org.eclipse.gef.NodeEditPart {

	public NodeEditPart(Node node) {
		setModel(node);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		Node node = (Node) getModel();
		switch (node.getType()) {
		case CIRCLE:
			return new CircleNodeFigure();
		case RECTANGLE:
			return new RectangleNodeFigure();
		}
		return null;
	}

	@Override
	protected void refreshVisuals() {
		AbstractNodeFigure figure = (AbstractNodeFigure) getFigure();
		Node node = (Node) getModel();
		GraphicalEditPart parent = (GraphicalEditPart) getParent();
		figure.getLabel().setText(node.getDescription());
		int x = (node.getX() >= 5) ? node.getX() : 10;
		int y = (node.getY() >= 5) ? node.getY() : 10;
		int width = (node.getWidth() >= 30) ? node.getWidth() : 30;
		int height = (node.getHeight() >= 30) ? node.getHeight() : 30;
		Rectangle r = new Rectangle(x, y, width, height);
		parent.setLayoutConstraint(this, figure, r);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new NodeGraphicalNodeEditPolicy());

	}

	@Override
	public Notifier getTarget() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean isAdapterForType(Object type) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void notifyChanged(Notification notification) {
//		int type = notification.getEventType();
//		switch (type) {
//		case Notification.ADD:
//		case Notification.ADD_MANY:
//		case Notification.REMOVE:
//		case Notification.REMOVE_MANY:
//			refreshChildren();
//			break;
//		case Notification.SET:
//			refreshVisuals();
//			break;
//		}
		int featureID = notification.getFeatureID(GraphPackage.class);
		switch(featureID) {
		case GraphPackage.NODE__OUTGOING_ARCS:
			refreshSourceConnections();
			break;
		case GraphPackage.NODE__INCOMING_ARCS:
			refreshTargetConnections();
			break;
		default:
			refreshVisuals();
			break;
		}
	}

	@Override
	public void setTarget(Notifier newTarget) {
		// TODO Auto-generated method stub

	}

	@Override
	public void activate() {
		if (!isActive()) {
			Node node = (Node) getModel();
			node.eAdapters().add(this);
			for (Arc arc : node.getIncomingArcs()) {
				arc.eAdapters().add(this);
			}
			for (Arc arc : node.getOutgoingArcs()) {
				arc.eAdapters().add(this);
			}
		}
		super.activate();
	}

	@Override
	public void deactivate() {
		if (isActive()) {
			Node node = (Node) getModel();
			node.eAdapters().remove(this);
			for (Arc arc : node.getIncomingArcs()) {
				arc.eAdapters().remove(this);
			}
			for (Arc arc : node.getOutgoingArcs()) {
				arc.eAdapters().remove(this);
			}
		}
		super.deactivate();
	}

	@Override
	protected List getModelSourceConnections() {
		return ((Node) getModel()).getOutgoingArcs();
	}

	@Override
	protected List getModelTargetConnections() {
		return ((Node) getModel()).getIncomingArcs();
	}

	@Override
	public ConnectionAnchor getSourceConnectionAnchor(
			ConnectionEditPart connection) {
		return ((AbstractNodeFigure) getFigure()).getConnectionAnchor();
	}

	@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return ((AbstractNodeFigure) getFigure()).getConnectionAnchor();
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(
			ConnectionEditPart connection) {
		return ((AbstractNodeFigure) getFigure()).getConnectionAnchor();
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return ((AbstractNodeFigure) getFigure()).getConnectionAnchor();
	}

}
